require 'spec_helper'

module Robot
  module Simulator
    describe Controller do
      context 'Robot simulator controller' do
        let!(:controller) { Controller.new }

        it 'controller should init the game with a robot and a 5 x 5 dimension table' do
          expect( controller.robot ).not_to be nil
          expect( controller.robot.table ).not_to be nil
          expect( controller.robot.table.dimensionX ).to eq(5)
          expect( controller.robot.table.dimensionY ).to eq(5)
        end

        context 'test place' do
          it 'controller should receive PLACE command and set robot at correct coordinate' do
            controller.place 0, 1, 'EAST'
            expect( controller.robot.current_track ).not_to be nil
            expect( controller.robot.current_track.coordinate.x ).to eq(0)
            expect( controller.robot.current_track.coordinate.y ).to eq(1)
          end

          it 'controller should raise error for invalid PLACE command' do
            expect{ controller.place 6, 6, 'EAST' }.to raise_error
          end
        end

        context 'test move' do
          it 'controller should receive MOVE command and move robot to correct coordinate' do
            controller.place 0, 1, 'EAST'
            controller.move
            expect( controller.robot.current_track.coordinate.x ).to eq(1)
            expect( controller.robot.current_track.coordinate.y ).to eq(1)
          end

          it 'controller should raise error for MOVE command before valid PLACE command' do
            expect{ controller.move }.to raise_error
          end

          it 'controller should ingore MOVE command if doing so will cause robot to fall' do
            controller.place 4, 4, 'EAST'
            expect{ controller.move }.to raise_error
            expect( controller.robot.current_track.coordinate.x ).to eq(4)
            expect( controller.robot.current_track.coordinate.y ).to eq(4)
          end
        end

        context 'test left' do
          it 'controller should receive LEFT command and change robot facing direction' do
            controller.place 0, 1, 'EAST'
            controller.left
            expect( controller.robot.current_track.facing ).to eq(Direction::NORTH)
            controller.left
            expect( controller.robot.current_track.facing ).to eq(Direction::WEST)
            controller.left
            expect( controller.robot.current_track.facing ).to eq(Direction::SOUTH)
            controller.left
            expect( controller.robot.current_track.facing ).to eq(Direction::EAST)
          end

          it 'controller should raise error for LEFT command before valid PLACE command' do
            expect{ controller.left }.to raise_error
          end
        end

        context 'test right' do
          it 'controller should receive RIGHT command and change robot facing direction' do
            controller.place 0, 1, 'EAST'
            controller.right
            expect( controller.robot.current_track.facing ).to eq(Direction::SOUTH)
            controller.right
            expect( controller.robot.current_track.facing ).to eq(Direction::WEST)
            controller.right
            expect( controller.robot.current_track.facing ).to eq(Direction::NORTH)
            controller.right
            expect( controller.robot.current_track.facing ).to eq(Direction::EAST)
          end

          it 'controller should raise error for RIGHT command before valid PLACE command' do
            expect{ controller.right }.to raise_error
          end
        end

        it 'test commands sequence' do
          controller.place 1, 2, 'EAST'
          controller.move
          controller.move
          controller.left
          controller.move
          expect( controller.report ).to eq [3, 3, "NORTH"]
        end

      end
    end
  end
end
