require 'spec_helper'

module Robot
  module Simulator
    describe Robot do

      let(:robot) { Robot.new Table.new(5, 5) }

      context 'test place method' do
        it 'expect robot current track to be nil before valid PLACE command' do
          expect( robot.current_track ).to be_nil
        end

        it 'expect robot current track NOT to be nil after valid PLACE command' do
          expect( robot.place(Coordinate.new(0, 0), Direction::EAST) ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.coordinate.x ).to eq(0)
          expect( robot.current_track.coordinate.y ).to eq(0)
          expect( robot.current_track.facing ).to eq(Direction::EAST)
        end

        it 'expect robot current track to be nil after invalid PLACE command' do
          expect( robot.place(Coordinate.new(6, 6), Direction::EAST) ).to be_falsey
          expect( robot.current_track ).to be_nil
        end

        it 'expect robot to stay on track and refuse subsequent invalid PLACE commands' do
          robot.send :current_track=, Track.new(Coordinate.new(0, 0), Direction::EAST)

          expect( robot.place(Coordinate.new(6, 6), Direction::EAST) ).to be_falsey
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.coordinate.x ).to eq(0)
          expect( robot.current_track.coordinate.y ).to eq(0)
          expect( robot.current_track.facing ).to eq(Direction::EAST)
        end
      end

      context 'test move method' do
        it 'expect robot move to be failed before valid PLACE command' do
          expect( robot.move() ).to be_falsey
          expect( robot.current_track ).to be_nil
        end

        it 'expect robot move to next track after valid PLACE command' do
          robot.send :current_track=, Track.new(Coordinate.new(0, 0), Direction::EAST)
          expect( robot.move() ).to be_truthy
          expect( robot.current_track.coordinate.x ).to eq(1)
          expect( robot.current_track.coordinate.y ).to eq(0)
        end

        it 'expect robot to refuse to move to prevent falling from table' do
          robot.send :current_track=, Track.new(Coordinate.new(4, 4), Direction::EAST)
          expect( robot.move() ).to be_falsey
          expect( robot.current_track.coordinate.x ).to eq(4)
          expect( robot.current_track.coordinate.y ).to eq(4)
        end
      end

      context 'test turn_left' do
        it 'expect robot to ingore LEFT command before valid PLACE command' do
          expect( robot.turn_left() ).to be_falsey
          expect( robot.current_track ).to be_nil
        end

        it 'expect robot to change facing direction when receive LEFT command' do
          robot.send :current_track=, Track.new(Coordinate.new(0, 0), Direction::EAST)

          expect( robot.turn_left() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::NORTH)

          expect( robot.turn_left() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::WEST)

          expect( robot.turn_left() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::SOUTH)

          expect( robot.turn_left() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::EAST)
        end
      end

      context 'test turn_right' do
        it 'expect robot to ingore RIGHT command before valid PLACE command' do
          expect( robot.turn_right() ).to be_falsey
          expect( robot.current_track ).to be_nil
        end

        it 'expect robot to change facing direction when receive RIGHT command' do
          robot.send :current_track=, Track.new(Coordinate.new(0, 0), Direction::EAST)

          expect( robot.turn_right() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::SOUTH)

          expect( robot.turn_right() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::WEST)

          expect( robot.turn_right() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::NORTH)

          expect( robot.turn_right() ).to be_truthy
          expect( robot.current_track ).not_to be_nil
          expect( robot.current_track.facing ).to eq(Direction::EAST)
        end
      end

    end
  end
end
