require 'spec_helper'

module Robot
  module Simulator
    module UI
      describe CLI do
        let(:cli) { CLI.new }

        it 'test commands from file' do
          expect( cli ).to receive(:say).with(/Reading robot commands from file/, anything)
          expect( cli ).to receive(:say).exactly 6
          expect( cli ).to receive(:say).with "3,3,NORTH"

          cli.options = { :file => File.expand_path('../commands.txt', __FILE__) }
          cli.start_game
        end

        context 'test commands from console' do
          let(:console_prompt) { "Please input robot commands, end with a blank line ..." }

          it "should process the commands and output the results" do
            expect( cli ).to receive(:say).with console_prompt, anything
            expect( cli ).to receive(:ask).with('>').and_return 'PLACE 1,2,East'
            expect( cli ).to receive(:ask).with('>').and_return 'MOVE'
            expect( cli ).to receive(:ask).with('>').and_return 'MOVE'
            expect( cli ).to receive(:ask).with('>').and_return 'LEFT'
            expect( cli ).to receive(:ask).with('>').and_return 'MOVE'
            expect( cli ).to receive(:ask).with('>').and_return 'REPORT'
            expect( cli ).to receive(:ask).with('>').and_return ''

            expect( cli ).to receive(:say).exactly 6
            expect( cli ).to receive(:say).with "3,3,NORTH"

            cli.start_game
          end

          context 'should report error for invalid commands' do
            it 'report unknow command error' do
              expect( cli ).to receive(:say).with console_prompt, anything
              expect( cli ).to receive(:ask).with('>').and_return 'PLACE2'
              expect( cli ).to receive(:ask).with('>').and_return ''

              expect( cli ).to receive(:say).once
              expect( cli ).to receive(:error).with "Unknow command is ignored: PLACE2"

              cli.start_game
            end

            it 'report error when there is exception' do
              expect( cli ).to receive(:say).with console_prompt, anything
              expect( cli ).to receive(:ask).with('>').and_return 'PLACE 1,2,EastWest'
              expect( cli ).to receive(:ask).with('>').and_return ''

              expect( cli ).to receive(:say).once
              expect( cli ).to receive(:error).with(/Direction string/)

              cli.start_game
            end

            it 'report error when robot would fall' do
              expect( cli ).to receive(:say).with console_prompt, anything
              expect( cli ).to receive(:ask).with('>').and_return 'PLACE 4,4,East'
              expect( cli ).to receive(:ask).with('>').and_return 'MOVE'
              expect( cli ).to receive(:ask).with('>').and_return ''

              expect( cli ).to receive(:say).twice
              expect( cli ).to receive(:error).with(/Invalid MOVE command/)

              cli.start_game
            end

            it 'report error if robot is not on the table' do
              expect( cli ).to receive(:say).with console_prompt, anything
              expect( cli ).to receive(:ask).with('>').and_return 'MOVE'
              expect( cli ).to receive(:ask).with('>').and_return 'LEFT'
              expect( cli ).to receive(:ask).with('>').and_return 'RIGHT'
              expect( cli ).to receive(:ask).with('>').and_return ''

              expect( cli ).to receive(:say).once
              expect( cli ).to receive(:error).with(/Invalid MOVE command/)

              expect( cli ).to receive(:say).once
              expect( cli ).to receive(:error).with(/Invalid LEFT command/)

              expect( cli ).to receive(:say).once
              expect( cli ).to receive(:error).with(/Invalid RIGHT command/)

              cli.start_game
            end

            context "should report invalid PLACE commands" do
              it 'report if PLACE on invalid position' do
                expect( cli ).to receive(:say).with console_prompt, anything
                expect( cli ).to receive(:ask).with('>').and_return 'PLACE'
                expect( cli ).to receive(:ask).with('>').and_return ''

                expect( cli ).to receive(:say).once
                expect( cli ).to receive(:error).with "Command PLACE is ignored because of lack parameters"

                cli.start_game
              end

              it 'report no arguments error' do
                expect( cli ).to receive(:say).with console_prompt, anything
                expect( cli ).to receive(:ask).with('>').and_return 'PLACE 5,5,EAST'
                expect( cli ).to receive(:ask).with('>').and_return ''

                expect( cli ).to receive(:say).once
                expect( cli ).to receive(:error).with(/Invalid PLACE command/)

                cli.start_game
              end

              it 'report no enough arguments error' do
                expect( cli ).to receive(:say).with console_prompt, anything
                expect( cli ).to receive(:ask).with('>').and_return 'PLACE 1,2'
                expect( cli ).to receive(:ask).with('>').and_return ''

                expect( cli ).to receive(:say).once
                expect( cli ).to receive(:error).with "Command PLACE needs 3 parameters in format of x,y,facing, got 1,2"

                cli.start_game
              end
            end
          end
        end

      end
    end
  end
end
